@@ -22,11 +22,14 @@ import pygments.lexers as pygments_lexers
 
 class IPythonPTCompleter(Completer):
     """Adaptor to provide IPython completions to prompt_toolkit"""
-    def __init__(self, ipy_completer=None, shell=None):
+    def __init__(self, ipy_completer=None, shell=None, patch_stdout=None):
         if shell is None and ipy_completer is None:
             raise TypeError("Please pass shell=an InteractiveShell instance.")
         self._ipy_completer = ipy_completer
         self.shell = shell
+        if patch_stdout is None:
+            raise TypeError("Please pass patch_stdout")
+        self.patch_stdout = patch_stdout
 
     @property
     def ipy_completer(self):
@@ -39,10 +42,14 @@ class IPythonPTCompleter(Completer):
         if not document.current_line.strip():
             return
 
-        used, matches = self.ipy_completer.complete(
-                            line_buffer=document.current_line,
-                            cursor_pos=document.cursor_position_col
-        )
+        # Some bits of our completion system may print stuff (e.g. if a module
+        # is imported). This context manager ensures that doesn't interfere with
+        # the prompt.
+        with self.patch_stdout():
+            used, matches = self.ipy_completer.complete(
+                                line_buffer=document.current_line,
+                                cursor_pos=document.cursor_position_col
+            )
         start_pos = -len(used)
         for m in matches:
             if not m:
